@@ -27,7 +27,7 @@ class InMemoryInsertOne:
     - Use add_record(record) to insert a new record; duplicates will raise an error.
     - Periodically call flush_all() to persist data and logs.
     """
-    
+
     def __init__(self, user_id, db, collection, container_url):
         self.user_id = user_id
         self.db = db
@@ -36,118 +36,99 @@ class InMemoryInsertOne:
         self.container_client = ContainerClient.from_container_url(container_url)
 
         self.shard_number = calculate_shard_number(user_id, db, collection, container_url)
-        self.shard_filename = f"{db}/{collection}/{collection}_{self.shard_number}.msgpack"
-        
-        # Use .msgpack instead of .json for index and logs
+        self.shard_filename = self._get_shard_filename(self.shard_number)
         self.index_filename = f"{db}/{collection}/index.msgpack"
         self.logs_filename = f"{db}/{collection}/logs.msgpack"
 
-        self._load_shard()
-        self._load_index()
-        self._load_logs()
+        self.shard_records = self._load_blob(self.shard_filename, default=[])
+        self.index_data = self._load_blob(self.index_filename, default={})
+        self.logs_data = self._load_blob(self.logs_filename, default={})
 
         self.shard_limit = get_shard_limit(user_id, db, collection, container_url)
-
         self.insert_count_since_flush = 0
-        self.flush_threshold = 1000
+        self.flush_threshold = 1
 
-        self.existing_ids = self._preload_existing_ids()
+        self.existing_ids = set()
+        self._populate_existing_ids()
 
-    def _load_shard(self):
-        blob_client = self.container_client.get_blob_client(self.shard_filename)
-        if blob_client.exists():
-            self.shard_records = msgpack.unpackb(blob_client.download_blob().readall(), raw=False)
-        else:
-            self.shard_records = []
+    def _get_shard_filename(self, shard_num):
+        return f"{self.db}/{self.collection}/{self.collection}_{shard_num}.msgpack"
 
-    def _load_index(self):
-        blob_client = self.container_client.get_blob_client(self.index_filename)
+    def _load_blob(self, filename, default):
         try:
-            index_bytes = blob_client.download_blob().readall()
-            self.index_data = msgpack.unpackb(index_bytes, raw=False)
+            blob = self.container_client.get_blob_client(filename)
+            return msgpack.unpackb(blob.download_blob().readall(), raw=False)
         except Exception:
-            self.index_data = []
+            return default
 
-    def _load_logs(self):
-        blob_client = self.container_client.get_blob_client(self.logs_filename)
-        try:
-            logs_bytes = blob_client.download_blob().readall()
-            self.logs_data = msgpack.unpackb(logs_bytes, raw=False)
-        except Exception:
-            self.logs_data = {}
-
-    def _preload_existing_ids(self):
-        ids = set()
-        for shard_map in self.index_data:
-            for shard_key, id_list in shard_map.items():
-                ids.update(id_list)
-        return ids
+    def get_last_inserted_id(self):
+        if self.index_data:
+            last_shard = f"{self.collection}_{self.shard_number}"
+            ids = self.index_data.get(last_shard, [])
+            if ids:
+                return ids[-1]
+        return None
 
     def add_log(self, method, log_type, log_message, count):
         log_id = str(uuid.uuid4())
         self.logs_data[log_id] = {
             "timestamp": datetime.datetime.now().isoformat(),
-            "method": method.upper(),
-            "type": log_type.upper(),
+            "method": method,
+            "type": log_type,
             "log": log_message,
             "count": count
         }
 
-    def add_record(self, record):
+    def _populate_existing_ids(self):
+        self.existing_ids = set()
+        for id_list in self.index_data.values():
+            self.existing_ids.update(id_list)
+
+    def add_record(self, record, force_flush=False):
         rec_id = record["_id"]
 
         if rec_id in self.existing_ids:
             raise ValueError(f"Duplicate _id detected: {rec_id}")
-
         self.existing_ids.add(rec_id)
 
         if len(self.shard_records) >= self.shard_limit:
             self.flush_shard()
             self.shard_number += 1
-            self.shard_filename = f"{self.db}/{self.collection}/{self.collection}_{self.shard_number}.msgpack"
+            self.shard_filename = self._get_shard_filename(self.shard_number)
             self.shard_records = []
 
         self.shard_records.append(record)
 
         shard_key = f"{self.collection}_{self.shard_number}"
-        found = False
-        for shard in self.index_data:
-            if shard_key in shard:
-                shard[shard_key].append(rec_id)
-                found = True
-                break
-        if not found:
-            self.index_data.append({shard_key: [rec_id]})
+
+        if shard_key in self.index_data:
+            self.index_data[shard_key].append(rec_id)
+        else:
+            self.index_data[shard_key] = [rec_id]
 
         self.insert_count_since_flush += 1
 
-        if self.insert_count_since_flush >= self.flush_threshold:
+        if force_flush or self.insert_count_since_flush >= self.flush_threshold:
             self.flush_all()
             self.insert_count_since_flush = 0
 
     def flush_all(self):
-        if self.shard_records or self.index_data or self.logs_data:
-            self.flush_shard()
-            self.flush_index()
-            self.flush_logs()
+        self.flush_shard()
+        self.flush_index()
+        self.flush_logs()
 
     def flush_shard(self):
-        blob_client = self.container_client.get_blob_client(self.shard_filename)
-        with BytesIO() as buf:
-            buf.write(msgpack.packb(self.shard_records, use_bin_type=True))
-            buf.seek(0)
-            blob_client.upload_blob(buf, overwrite=True)
+        self._flush_blob(self.shard_filename, self.shard_records)
 
     def flush_index(self):
-        blob_client = self.container_client.get_blob_client(self.index_filename)
-        with BytesIO() as buf:
-            buf.write(msgpack.packb(self.index_data, use_bin_type=True))
-            buf.seek(0)
-            blob_client.upload_blob(buf, overwrite=True)
+        self._flush_blob(self.index_filename, self.index_data)
 
     def flush_logs(self):
-        blob_client = self.container_client.get_blob_client(self.logs_filename)
+        self._flush_blob(self.logs_filename, self.logs_data)
+
+    def _flush_blob(self, filename, data):
+        blob = self.container_client.get_blob_client(filename)
         with BytesIO() as buf:
-            buf.write(msgpack.packb(self.logs_data, use_bin_type=True))
+            buf.write(msgpack.packb(data, use_bin_type=True))
             buf.seek(0)
-            blob_client.upload_blob(buf, overwrite=True)
+            blob.upload_blob(buf, overwrite=True)
