@@ -14,20 +14,16 @@ class InMemoryInsertMany:
         self.container_url = container_url
         self.container_client = ContainerClient.from_container_url(container_url)
 
-        # Load only the last shard number
         self.shard_number = self._get_last_shard_number()
         self.shard_filename = f"{db}/{collection}/{collection}_{self.shard_number}.msgpack"
-
         self.index_filename = f"{db}/{collection}/index.msgpack"
         self.logs_filename = f"{db}/{collection}/logs.msgpack"
-
         self.shard_limit = get_shard_limit(user_id, db, collection, container_url)
 
         self._load_shard()
         self._load_index()
         self._load_logs()
 
-        # Only track existing IDs from current shard and index
         self.existing_ids = set()
         self._initialize_existing_ids()
 
@@ -48,9 +44,10 @@ class InMemoryInsertMany:
     def _load_index(self):
         blob_client = self.container_client.get_blob_client(self.index_filename)
         try:
-            self.index_data = msgpack.unpackb(blob_client.download_blob().readall(), raw=False)
+            data = msgpack.unpackb(blob_client.download_blob().readall(), raw=False)
+            self.index_data = data if isinstance(data, dict) else {}
         except Exception:
-            self.index_data = []
+            self.index_data = {}
 
     def _load_logs(self):
         blob_client = self.container_client.get_blob_client(self.logs_filename)
@@ -60,15 +57,22 @@ class InMemoryInsertMany:
             self.logs_data = {}
 
     def _initialize_existing_ids(self):
-        # Only gather IDs from current shard and current shard entry in index
+        self.existing_ids.clear()
         for rec in self.shard_records:
             if "_id" in rec:
                 self.existing_ids.add(rec["_id"])
 
         shard_key = f"{self.collection}_{self.shard_number}"
-        for shard_map in self.index_data:
-            if shard_key in shard_map:
-                self.existing_ids.update(shard_map[shard_key])
+        if shard_key in self.index_data:
+            self.existing_ids.update(self.index_data[shard_key])
+
+    def get_last_id(self):
+        shard_key = f"{self.collection}_{self.shard_number}"
+        if shard_key in self.index_data and self.index_data[shard_key]:
+            return self.index_data[shard_key][-1]
+        elif self.shard_records:
+            return self.shard_records[-1].get("_id")
+        return None
 
     def add_log(self, method, log_type, log_message, count):
         log_id = str(uuid.uuid4())
@@ -86,18 +90,13 @@ class InMemoryInsertMany:
             self.shard_number += 1
             self.shard_filename = f"{self.db}/{self.collection}/{self.collection}_{self.shard_number}.msgpack"
             self.shard_records = []
-            # Reset ID tracking for new shard
-            self.existing_ids.clear()
+            self._initialize_existing_ids() 
 
         self.shard_records.append(record)
         self.existing_ids.add(record["_id"])
 
         shard_key = f"{self.collection}_{self.shard_number}"
-        for shard in self.index_data:
-            if shard_key in shard:
-                shard[shard_key].append(record["_id"])
-                return
-        self.index_data.append({shard_key: [record["_id"]]})
+        self.index_data.setdefault(shard_key, []).append(record["_id"])
 
     def flush_all(self):
         self.flush_shard()
@@ -123,4 +122,4 @@ class InMemoryInsertMany:
         with BytesIO() as buf:
             buf.write(msgpack.packb(self.logs_data, use_bin_type=True))
             buf.seek(0)
-            blob_client.upload_blob(buf, overwrite=True)
+            blob_client.upload_blob(buf, overwrite=True)
\ No newline at end of file
