@@ -13,15 +13,12 @@
 # limitations under the License.
 # Visit www.erioon.com/dev-docs for more information about the python SDK
 
-import uuid
 from erioon.InMemoryinsertMany import InMemoryInsertMany
 from erioon.InMemoryInsertOne import InMemoryInsertOne
 from erioon.functions import generate_next_id
 
 # INSERT ONE RECORD
 user_contexts = {}
-
-
 def handle_insert_one(user_id_cont, database, collection, record, container_url):
     """
     Insert a single record into the collection.
@@ -30,7 +27,7 @@ def handle_insert_one(user_id_cont, database, collection, record, container_url)
     - If provided '_id' is duplicate, generate a new one and update the record.
     - Create or append the record in a shard file.
     - Update index.msgpack to map the record to the appropriate shard.
-    - Log success or errors asynchronously.
+    - Log success or errors.
 
     Args:
         user_id_cont: User identifier.
@@ -52,20 +49,17 @@ def handle_insert_one(user_id_cont, database, collection, record, container_url)
 
     try:
         last_id = None
-        # Get last _id from the index if exists
         if context.index_data:
-            last_shard = context.index_data[-1]  # last shard dict
-            last_id_lists = list(last_shard.values())[0] if last_shard else []
-            if last_id_lists:
-                last_id = last_id_lists[-1]
+            last_shard_key = sorted(context.index_data.keys())[-1]
+            last_id_list = context.index_data.get(last_shard_key, [])
+            if last_id_list:
+                last_id = last_id_list[-1]
 
-        # Use shard_number from context
         new_id = generate_next_id(last_id, context.shard_number)
 
         if "_id" not in record or not record["_id"]:
             record["_id"] = new_id
         else:
-            # Check duplicate _id, replace with new generated if duplicate
             if record["_id"] in context.existing_ids:
                 record["_id"] = new_id
 
@@ -88,20 +82,11 @@ def handle_insert_one(user_id_cont, database, collection, record, container_url)
             "error": str(e),
         }, 500
 
-
 # INSERT MANY RECORDS
 def handle_insert_many(user_id_cont, database, collection, data, container_url):
     """
     Insert multiple records in bulk.
 
-    - `data` is a list of dicts, each representing a record.
-    - For each record:
-      - Ensure it has a unique _id (generate new UUID if missing or duplicate).
-      - Write the record to the appropriate shard.
-      - Update index.msgpack with _id to shard mapping.
-    - Log the batch insert operation with details.
-    - Return aggregate success or failure response.
-
     Args:
         user_id_cont: User identifier.
         database: Database name.
@@ -116,19 +101,11 @@ def handle_insert_many(user_id_cont, database, collection, data, container_url):
     insert_results = []
 
     try:
-        # Get last _id from index if exists
-        last_id = None
-        if context.index_data:
-            last_shard = context.index_data[-1]
-            last_id_list = list(last_shard.values())[0] if last_shard else []
-            if last_id_list:
-                last_id = last_id_list[-1]
+        last_id = context.get_last_id()
 
         for record in data:
-            new_id = generate_next_id(last_id, context.shard_number)
-
             if "_id" not in record or not record["_id"]:
-                record["_id"] = new_id
+                record["_id"] = generate_next_id(last_id, context.shard_number)
             else:
                 while record["_id"] in context.existing_ids:
                     record["_id"] = generate_next_id(last_id, context.shard_number)
